"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.spreadMatcher = void 0;
const lodash_1 = require("lodash");
const spreadMatcher = (originalData) => {
    const data = lodash_1.cloneDeep(originalData);
    const matches = [];
    let i = 0;
    while (true) {
        if (i > 10000) {
            throw new Error(`
              Matching executed more than 10000 - it may mean, that something broke
              data: ${JSON.stringify(data, null, 2)}
              matches: not displayed, because of potential length
            `);
        }
        const allGroupsHaveVolume = [
            data.entityGroups[0].some((v) => v.volume > 0),
            data.entityGroups[1].some((v) => v.volume > 0)
        ].every(Boolean);
        if (!allGroupsHaveVolume) {
            break;
        }
        const roundMatches = matchRound(getRoundInput(data));
        roundMatches.forEach((match) => {
            data.entityGroups[0].find((c) => c.id === match.entities[0]).volume -= match.volume;
            data.entityGroups[1].find((g) => g.id === match.entities[1]).volume -= match.volume;
        });
        matches.push(roundMatches);
        i += 1;
    }
    return {
        matches: sumMatches(matches.flat()),
        leftoverEntities: data.entityGroups
    };
};
exports.spreadMatcher = spreadMatcher;
const sumMatches = (matches) => {
    return Object.values(lodash_1.groupBy(matches, (m) => `${m.entities[0]}${m.entities[1]}`)).map((matches) => ({
        entities: matches[0].entities,
        volume: lodash_1.sumBy(matches, (m) => m.volume)
    }));
};
const computeSafeDistribution = (entities1, entities2) => {
    const compute = (entities, otherEntitiesLength) => entities.map((entity) => ({
        entityId: entity.id,
        volume: entity.volume,
        distributedVolume: Math.floor(entity.volume / otherEntitiesLength)
    }));
    return lodash_1.minBy([...compute(entities1, entities2.length), ...compute(entities2, entities1.length)], (e) => e.distributedVolume);
};
const getCompetingEntites = (routes, entityId) => {
    return routes
        .filter((route) => route.includes(entityId))
        .flat()
        .filter((id) => id !== entityId);
};
const matchRound = (input) => {
    const entityToDistribute = computeSafeDistribution(input.entityGroups[0], input.entityGroups[1]);
    if (entityToDistribute.distributedVolume === 0) {
        const competetingEntities = getCompetingEntites(input.routes, entityToDistribute.entityId);
        const entitiesCountToUse = entityToDistribute.volume;
        const isEntityFirst = input.entityGroups[0].find((e) => e.id === entityToDistribute.entityId);
        return competetingEntities.slice(0, entitiesCountToUse).map((connectedEntity) => ({
            entities: isEntityFirst
                ? [entityToDistribute.entityId, connectedEntity]
                : [connectedEntity, entityToDistribute.entityId],
            volume: 1
        }));
    }
    return input.routes.map((route) => ({
        entities: route,
        volume: entityToDistribute.distributedVolume
    }));
};
const getRoundInput = (data) => {
    const entity1round = getFirstUsableGroup(data.priority, data.entityGroups[0]);
    const routes = entity1round.flatMap((entity1) => {
        const prefferedEntity2Group = getFirstUsableGroup(entity1.priority, data.entityGroups[1]);
        return prefferedEntity2Group.map((entity2) => [entity1.id, entity2.id]);
    });
    return {
        routes,
        entityGroups: [
            data.entityGroups[0].filter((entity) => routes.some((e) => e[0] === entity.id)),
            data.entityGroups[1].filter((entity) => routes.some((e) => e[1] === entity.id))
        ]
    };
};
const getFirstUsableGroup = (groups, data) => {
    const groupsWithVolume = groups
        .map((group) => {
        const withVolume = group.map((entry) => ({
            ...entry,
            volume: data.find((c) => c.id === entry.id)?.volume ?? 0
        }));
        return withVolume.filter((entry) => entry.volume > 0);
    })
        .filter((group) => group.length > 0);
    return groupsWithVolume[0] ?? [];
};
//# sourceMappingURL=spreadMatcher.js.map